/*++

Copyright (c) Sydus.

Module:

    Selector_Poll_Posix.h

Authors:

    Sharvil Nanavati (sharvil) 2006-09-26

--*/

#pragma once

#include "base.h"
#include "ISelectorImplementation.h"

#include <vector>
#include <sys/poll.h>

class Selector_Poll_Posix : public ISelectorImplementation
{
	public:
		Selector_Poll_Posix();
		~Selector_Poll_Posix();

		virtual void Alert();

		virtual int32 Select();
		virtual int32 Select(uint32 timeoutMillis);

		virtual const SelectedSet & AcceptSet() const;
		virtual const SelectedSet & ReadSet() const;
		virtual const SelectedSet & WriteSet() const;

		virtual bool IsAlerted() const;

	private:
		virtual bool Register(int32 fd, ISelectable & item);
		virtual void Unregister(int32 fd);

		typedef std::vector <pollfd> FdList;
		typedef std::vector <ISelectable *> SelectableList;

		//
		// Note: there is an implicit association between entries in these two
		// vectors. That is, iFileDescriptors[n] <--> iSelectables[n] for all
		// 0 <= n < iFileDescriptors.length().
		//
		FdList iFileDescriptors;
		SelectableList iSelectables;

		SelectedSet iAcceptReady;
		SelectedSet iReadReady;
		SelectedSet iWriteReady;

		bool  iIsAlerted;       // Whether alert was called.
		int32 iAlertWrite;      // The write end of the pipe used to alert
		int32 iAlertRead;       // The read end of the pipe used to alert
};
