/*++

Copyright (c) Sydus.

Module:

    Selector_Posix.h

Authors:

    Sharvil Nanavati (sharvil) 2005-01-08

--*/

#pragma once

#include "base.h"
#include "ISelectorImplementation.h"

#include <ext/hash_map>
using __gnu_cxx::hash_map;

class Selector_Posix : public ISelectorImplementation
{
	public:
		Selector_Posix();
		~Selector_Posix();

		virtual void Alert();

		virtual int32 Select();
		virtual int32 Select(uint32 timeoutMillis);

		virtual const SelectedSet & AcceptSet() const;
		virtual const SelectedSet & ReadSet() const;
		virtual const SelectedSet & WriteSet() const;

		virtual bool IsAlerted() const;

	private:
		virtual bool Register(int32 fd, ISelectable & item);
		virtual void Unregister(int32 fd);

		typedef hash_map <int32, ISelectable *> FdMap;

		FdMap iSelectables;
		SelectedSet iAcceptReady;
		SelectedSet iReadReady;
		SelectedSet iWriteReady;

		bool  iIsAlerted;       // Whether alert was called.
		int32 iAlertWrite;      // The write end of the pipe used to alert
		int32 iAlertRead;       // The read end of the pipe used to alert
};
