/*++

Copyright (c) Sydus.

Module:

	Process_Posix.h

Authors:

	Serge Zloto (szloto) 2006-09-27

--*/

#pragma once

#include "base.h"

#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <string>

/**
 * Process is an (poor) abstraction for manipulating, creating and comparing
 * operating system processes.
 * Ultimately this will turn into a real abstraction, with a process factory
 * and real process objects...
 */
class Process_Posix: public NonCopyable
{
public:

	typedef uint16 Id;

	/**
	 * Sets up the stage for functionality such as altering the process name
	 * as seen by ps(1).
	 * Should be called only once for the process lifetime
	 */
	static void Initialize(char* argv);

	/**
	 * Gets an identifier for the current process
	 */
	static Id CurrentId();

	/**
	 * @return the process name
	 */
	static const char* GetName();

	/**
	 * Sets the process name
	 */
	static void SetName(const std::string& name);

	/**
	 * Creates a copy of the current process
	 * @return 0 in the child process, the child's Id in the parent process
	 * @throws exception if Fork fails. No process is created then
	 */
	static Id Fork();

	/**
	 * Wait for any child process to end
	 */
	static Id Wait();
	
	/**
	 * Bounded wait for any child process to end
	 */
	static Id Wait(uint32 milliseconds);

private:
	/** Catches the dead processes */
	static void CatchDeadProcess(int sig);

	/** Disallowed **/
	Process_Posix();

	~Process_Posix();

	/**
	 * Equality
	 */
	bool operator==(const Process_Posix& other) const;

	static char* iArgv0;
};

