import os, sys, glob

mingw = int(ARGUMENTS.get('mingw', 0))
debug = int(ARGUMENTS.get('debug', 1))
wine = int(ARGUMENTS.get('wine', 0))

isGcc = wine or mingw or os.name == 'posix'

if os.name == 'nt':
	basepath = '/Projects/'
else:
	basepath = os.environ['HOME'] + '/projects/'

corelibpath = basepath + 'corelib/'

# ------------------------------------------------------- Environment Routines

def ImportVariable(env, name):
	if name in os.environ:
		env['ENV'][name] = os.environ[name]

def ImportDirect(env, name):
	if name in os.environ:
		env[name] = os.environ[name]

def InitializePlatformEnvironment(env):
	if isGcc:
		ImportVariable(env, 'TERM')
		ImportVariable(env, 'HOME')
		ImportDirect(env, 'CC')
		ImportDirect(env, 'CXX')
		env['CCFLAGS'] = ['-Wall', '-Werror']
	else:
		env['CCFLAGS'] = ['/W3', '/WX', '/EHsc', '/wd4355', '/wd4996', '/wd4244', '/nologo', '/Gd']
		env['LINKFLAGS'] = ['/SUBSYSTEM:CONSOLE', '/nologo']

	if os.name == 'nt':
		env['LIBS'] = ['ws2_32', 'advapi32']
	else:
		env['LIBS'] = ['pthread', 'dl']

def DebugEnvironment(env):
	if isGcc:
		env.Append(CCFLAGS = ['-g'])
	else:
		env.Append(CCFLAGS = ['/Zi', '/Od', '/RTC1', '/MDd'])

def ReleaseEnvironment(env):
	if isGcc:
		env.Append(CCFLAGS = ['-O2'])
	else:
		env.Append(CCFLAGS = ['/O2', '/MD'])

# --------------------------------------------------------------- Build Script

if mingw:
	env = Environment(tools = ['mingw'])
else:
	env = Environment()

InitializePlatformEnvironment(env)

#
# Add debug/optimization flags depending on whether debug=1 was specified.
#
if debug:
	DebugEnvironment(env)
else:
	ReleaseEnvironment(env)

if wine:
	env['CC'] = 'winegcc'
	env['CXX'] = 'wineg++'
	os.name = 'nt'

#
# Add the library paths.
#
env.Append(CPPPATH = [corelibpath + 'contrib', corelibpath + 'include', 'include', 'include/platform/' + os.name])
env.Append(LIBPATH = [corelibpath + 'lib'])
